@@ -9,6 +9,7 @@
 
 import enum
 import logging
+import platform
 import sys
 from enum import Enum
 from typing import List, Optional
@@ -20,7 +21,7 @@ from pynitrokey.fido2 import device_path_to_str
 
 from .base import Nitrokey3Base
 from .exceptions import TimeoutException
-from .utils import Version
+from .utils import Uuid, Version
 
 RNG_LEN = 57
 UUID_LEN = 16
@@ -40,6 +41,7 @@ class Command(Enum):
     UUID = 0x62
     LOCKED = 0x63
     OTP = 0x70
+    PROVISIONER = 0x71
 
 
 @enum.unique
@@ -95,14 +97,14 @@ class Nitrokey3Device(Nitrokey3Base):
             self.logger.debug("ignoring OSError after reboot", exc_info=e)
         return True
 
-    def uuid(self) -> Optional[int]:
+    def uuid(self) -> Optional[Uuid]:
         uuid = self._call(Command.UUID)
         if len(uuid) == 0:
             # Firmware version 1.0.0 does not support querying the UUID
             return None
         if len(uuid) != UUID_LEN:
             raise ValueError(f"UUID response has invalid length {len(uuid)}")
-        return int.from_bytes(uuid, "big")
+        return Uuid(int.from_bytes(uuid, byteorder="big"))
 
     def version(self) -> Version:
         version_bytes = self._call(Command.VERSION, response_len=VERSION_LEN)
@@ -147,7 +149,10 @@ class Nitrokey3Device(Nitrokey3Base):
     @staticmethod
     def open(path: str) -> Optional["Nitrokey3Device"]:
         try:
-            device = open_device(path)
+            if platform.system() == "Windows":
+                device = open_device(bytes(path, "utf-8"))
+            else:
+                device = open_device(path)
         except Exception:
             logger.warn(f"No CTAPHID device at path {path}", exc_info=sys.exc_info())
             return None
